from string import atoi, atof
from orchestra import Note, NoteList

class ParseState:
  def __init__(self, noteModifiers):
    self.octave = 8
    self.scaleDegree = 0
    self.dur = 1.0
    self.amp = -12.0
    self.start = 0.0
    self.articulation = 0
    self.noteModifiers = noteModifiers
    self.currentModifier = None
  
  def parseVal(self,val,state):
    try:
      if(state == 0):
        parts = val.split('.')
        if(len(parts) == 2):
          self.octave = atoi(parts[0])
          self.scaleDegree = atoi(parts[1])
        else:
          self.scaleDegree = atoi(val)
        
      elif(state == 1):
        self.dur = atof(val)
    
      elif(state == 2):
        self.amp = atof(val) 

      elif(state == 3):
        self.articulation = atoi(val)

      elif(state == 4):
        self.start += atof(val)

    except Exception, e:
      raise Exception("Error parsing value '%s', cause: %s"%(val, e.message))

  def parseNoteModifier(self,modStr):
     if(modStr == 'clear'):
       self.currentModifier = None
     else:
       if(self.noteModifiers.has_key(modStr)):
         self.currentModifier = self.noteModifiers[modStr]
       else:
         raise Exception("No modifier found with name '%s'"%modStr)
 
  def parseFragment(self,noteStr):
    val = ""
    state = 0
    scaleDegree = -1

    if(noteStr[0] == 'r'):
      self.parseVal(noteStr[1:], 4) 
      return None

    if(noteStr[0] == 'm'):
      if(noteStr[1] != ':'):
        raise Exception("Error with m command: m must be followed by :")
      self.parseNoteModifier(noteStr[2:])
      return None

    for c in noteStr:
       if(c.isalpha() or c == ':'):
         self.parseVal(val, state)
         val = ""
         if(c == 'd'):
           state = 1
         elif(c == 'a'):
           state = 2 
         elif(c == ':'):
           state = 3
         else:
           raise Exception("Unknown note modifier character '%s', found in '%s'"%(c,noteStr))
       else:
         val += c
    self.parseVal(val, state)
    start = self.start
    self.start += abs(self.dur)
    #return (start, self.dur, self.octave, self.scaleDegree, self.amp, self.articulation) 
    pch = "%d.%02d" % (self.octave, self.scaleDegree)
    n = Note(start, self.dur, pch, pch, self.amp, self.articulation)

    if(self.currentModifier != None):
      self.currentModifier(n)   

    return n
    #return NoteList([ Note(0, 1, pch, pch, amp, 1) ])

def parseOrchScore(scoreString,noteModifiers=None):
  """Parses an Orchestra Format Note String into a list of tuples.

  The OrchNote short format allows text  in the form of:

  8.00d-.25a-12:1

  which would mean octave 8, scaleDegree 0, duration -.25 (tied note), amplitude -12, and 
  articulation 1. Future notes may skip the octave, duration, and amplitude, as those will 
  automatically get carried over. Rests may also be used by using an 'r' followed by a 
  duration, for example:
  
  r2

  would rest for 2 beats. The result of noteParse returns a NoteList of orchestra Note objects.
   
  Note modifiers can be used by passing in a map of modifiers names : modifier functions, 
  where the functions are single-arg functions that take in Note objects to process.

  Example:

  def stoccato(n):
    n.duration = n.duration * .5

  modifiers = {"stoccato": stoccato }

  a = "m:stoccato 8.00d.25a-12 4 m:clear 3d.5 2 1 0d1a-8"
  notes = parseOrchScore(a, modifiers)

  score = groupB64[0].perform(notes)

"""

  
  parts = scoreString.split()

  #retVal = []
  retVal = NoteList()
 
  if(len(parts) == 0):
    return retVal

  state = ParseState(noteModifiers)

  for i in parts:
    val = state.parseFragment(i)
    if(val != None):
      retVal.append(val) 

  return retVal


